@@ -1068,6 +1068,14 @@ GIFEncode(FILE * const fp,
     int const TopOfs = 0;
     int InitCodeSize;
     
+    if (GWidth > 65535)
+        pm_error("Image width %u too large for GIF format.  (Max 65535)",
+                 GWidth);  
+    
+    if (GHeight > 65535)
+        pm_error("Image height %u too large for GIF format.  (Max 65535)",
+                 GHeight);  
+
     writeGifHeader(fp, GWidth, GHeight, GInterlace, Background,
                    BitsPerPixel, cmapP, comment);
 
