@@ -724,7 +724,7 @@ main(argc, argv)
                 if( colormap ) {
                     cmapmaxval = maxval;
                     pm_message("%d colors found", colors);
-                    nPlanes = pm_maxvaltobits(colors);
+                    nPlanes = pm_maxvaltobits(colors-1);
                     if( fixplanes > nPlanes )
                         nPlanes = fixplanes;
                 }
