@@ -48,7 +48,13 @@
  * HEADER FILES *
  *==============*/
 
+#include <assert.h>
+#include <sys/types.h>
 #include <sys/times.h>
+
+#include "ppm.h"
+#include "nstring.h"
+
 #include "all.h"
 #include "mtypes.h"
 #include "bitio.h"
@@ -83,10 +89,13 @@ static int	RateControlMode = VARIABLE_RATE;
 static int32 buffer_size = DEFAULT_BUFFER_SIZE;
 static int32 bit_rate = -1;
 
+static bool wantVbvUnderflowWarning;
+static bool wantVbvOverflowWarning;
 
 /*   Variables for the VBV buffer defined in MPEG specs */
-static int32 VBV_delay =0;	    /* delay in units of 1/90000 seconds */
-static int32 VBV_buffer = 0;	    /* fullness of the theoretical VBV buffer */
+static unsigned int VBV_remainingDelay; 
+    /* delay in units of 1/90000 seconds */
+static int32 VBV_buffer = 0;	  /* fullness of the theoretical VBV buffer */
 static int32 bufferFillRate = 0;    /* constant rate at which buffer filled */
 static int32 frameDelayIncrement = 0;	/* number of "delay" units/Frame */
 
@@ -103,10 +112,10 @@ int Ti, Tp, Tb;
 int current_Tx;	/* allocation for current frame */
 
 /*  Count of number of pictures of each type remaining */
-int GOP_X = 0;
-int GOP_I = 0;
-int GOP_P = 0;
-int GOP_B = 0;
+int GOP_X;
+int GOP_I;
+int GOP_P;
+int GOP_B;
 
 int Nx = 0;
 int Ni = 0;
@@ -197,111 +206,133 @@ extern int framePatternLen;
  *===============================*/
 
 int initGOPRateControl _ANSI_ARGS_((void));
-     int determineMBCount _ANSI_ARGS_((void));
-     void checkBufferFullness _ANSI_ARGS_((int count));
-     void checkSpatialActivity _ANSI_ARGS_((Block blk0, Block blk1, Block blk2, Block blk3));
-     void incNumBlocks _ANSI_ARGS_((int num));
-     void calculateVBVDelay _ANSI_ARGS_((int num));
-     void updateVBVBuffer _ANSI_ARGS_((int frameBits));
-     int BlockExperiments  _ANSI_ARGS_((int16 *OrigBlock, int16 *NewBlock, int control));
+int determineMBCount _ANSI_ARGS_((void));
+void checkBufferFullness _ANSI_ARGS_((int count));
+void checkSpatialActivity _ANSI_ARGS_((Block blk0, Block blk1, Block blk2, Block blk3));
+void incNumBlocks _ANSI_ARGS_((int num));
+void calculateVBVDelay _ANSI_ARGS_((int num));
+int BlockExperiments  _ANSI_ARGS_((int16 *OrigBlock, int16 *NewBlock, int control));
      
      
-     /*=====================*
-      * EXPORTED PROCEDURES *
-      *=====================*/
-     
-     /*===========================================================================*
-      *
-      * initRateControl
-      *
-      *	initialize the allocation parameters.
-      *
-      * RETURNS:	nothing
-      *
-      * SIDE EFFECTS:   many global variables 
-      *
-      * NOTES:  Get rid of the redundant pattern stuff!!
-      *===========================================================================*/
-     int
-       initRateControl()
-{
-  int index;
-  int result;
+
+static void
+analyzePattern(const char *  const framePattern,
+               int           const framePatternLen,
+               int *         const gop_xP,
+               int *         const gop_iP,
+               int *         const gop_pP,
+               int *         const gop_bP,
+               const char ** const errorP) {
+
+    unsigned int i;
+
+    /*  Initialize Pattern info */
+    *gop_xP = framePatternLen;    
+
+    for (i = 0, *gop_iP = 0, *gop_pP = 0, *gop_bP = 0, *errorP = NULL;
+         i < framePatternLen && !*errorP; 
+         ++i) {
+        switch(framePattern[i]) {
+        case 'i': ++*gop_iP; break;
+        case 'p': ++*gop_pP; break;
+        case 'b': ++*gop_bP; break;
+        default:
+            asprintfN(errorP, "Bad pattern - not composed of i, p, and b");
+        }
+    }
+    assert(*gop_xP == *gop_iP + *gop_pP + *gop_bP);
+}
   
-  DBG_PRINT(("\tInitializing Allocation Data\n"));
+
+
+/*===========================================================================*
+ *
+ * initRateControl
+ *
+ *	initialize the allocation parameters.
+ *
+ * RETURNS:	nothing
+ *
+ * SIDE EFFECTS:   many global variables 
+ *
+ * NOTES:  Get rid of the redundant pattern stuff!!
+ *===========================================================================*/
+int
+initRateControl(bool const wantUnderflowWarning,
+                bool const wantOverflowWarning) {
+    int result;
+    const char * error;
+
+    wantVbvUnderflowWarning = wantUnderflowWarning;
+    wantVbvOverflowWarning  = wantOverflowWarning;
+
+    DBG_PRINT(("Initializing Allocation Data\n"));
   
 #ifdef RC_STATS_FILE
-  RC_FILE = fopen("RC_STATS_FILE", "w");
-  if ( RC_FILE  == NULL) {
-    DBG_PRINT(("\tOpen of RC file failed, using stderr\n"));
-    RC_FILE = stderr;
-    fprintf(RC_FILE, "\tOpen of RC file failed, using stderr\n");
-    fflush(RC_FILE);
-  }
-#endif
-  
-  /*  Initialize Pattern info */
-  GOP_X = framePatternLen;
-  for ( index = 0; index < framePatternLen; index++ ) {
-    switch( framePattern[index] ) {
-    case 'i':
-      GOP_I++;
-      break;
-    case 'p':
-      GOP_P++;
-      break;
-    case 'b':
-      GOP_B++;
-      break;
-    default:
-      printf("\n\tERROR rate.c - BAD PATTERN!\n");
-      RateControlMode = VARIABLE_RATE;
-      return (0);
+    RC_FILE = fopen("RC_STATS_FILE", "w");
+    if ( RC_FILE  == NULL) {
+        DBG_PRINT(("Open of RC file failed, using stderr\n"));
+        RC_FILE = stderr;
+        fprintf(RC_FILE, "Open of RC file failed, using stderr\n");
+        fflush(RC_FILE);
     }
-  }
-  if (GOP_X != (GOP_I + GOP_P + GOP_B )) {
-    printf("\n\tERROR rate.c - Pattern Length Mismatch\n");
-    RateControlMode = VARIABLE_RATE;
-    return (-1);
-  }
+#endif
+
+    VBV_remainingDelay = 0;
   
-  /* Initializing GOP bit allocation */	
-  rc_R = 0;
-  rc_G = (bit_rate * GOP_X/frameRateRounded);
+    analyzePattern(framePattern, framePatternLen,
+                   &GOP_X, &GOP_I, &GOP_P, &GOP_B, &error);
+
+    if (error) {
+        pm_message("Unable to set up rate control.  Switching to variable.  "
+                   "%s", error);
+        strfree(error);
+        RateControlMode = VARIABLE_RATE;
+        return -1;
+    }
+
+
+    /* Initializing GOP bit allocation */	
+    rc_R = 0;
+    rc_G = (bit_rate * GOP_X/frameRateRounded);
   
-  /*   Initialize the "global complexity measures" */
-  Xi = (160 * bit_rate/115);
-  Xp = (60 * bit_rate/115);
-  Xb = (42 * bit_rate/115);
+    /*   Initialize the "global complexity measures" */
+    Xi = (160 * bit_rate/115);
+    Xp = (60 * bit_rate/115);
+    Xb = (42 * bit_rate/115);
   
-  /*   Initialize MB counters */
-  rc_totalMBBits= rc_bitsThisMB= rc_totalFrameBits=rc_totalOverheadBits = 0;
-  rc_numBlocks = rc_totalQuant = 0;
+    /*   Initialize MB counters */
+    rc_totalMBBits= rc_bitsThisMB= rc_totalFrameBits=rc_totalOverheadBits = 0;
+    rc_numBlocks = rc_totalQuant = 0;
   
-  /*   init virtual buffers  */
-  reactionParameter = (2 * bit_rate / frameRateRounded);
-  d0_i = (10 * reactionParameter / 31);
-  d0_p = (Kp * d0_i);
-  d0_b = (Kb * d0_i);
+    /*   init virtual buffers  */
+    reactionParameter = (2 * bit_rate / frameRateRounded);
+    d0_i = (10 * reactionParameter / 31);
+    d0_p = (Kp * d0_i);
+    d0_b = (Kb * d0_i);
   
-  lastFrameVirtBuf = d0_i;	/*  start with I Frame */
-  rc_Q = lastFrameVirtBuf  * 31 / reactionParameter;
+    lastFrameVirtBuf = d0_i;	/*  start with I Frame */
+    rc_Q = lastFrameVirtBuf  * 31 / reactionParameter;
   
-  /*   init spatial activity measures */
-  avg_act = 400;		/* Suggested initial value */
-  N_act = 1;
+    /*   init spatial activity measures */
+    avg_act = 400;		/* Suggested initial value */
+    N_act = 1;
   
-  mquant = rc_Q * N_act;
+    mquant = rc_Q * N_act;
   
-  frameDelayIncrement = (90000 / frameRateRounded); /* num of "delay" units per frame */
-  bufferFillRate = bit_rate / frameRateRounded; /* VBV buf fills at constant rate */
-  VBV_buffer = buffer_size;
-  DBG_PRINT(("\tVBV- delay: %d, fill rate: %d, delay/Frame: %d units, buffer size: %d\n",
-	     VBV_delay, bufferFillRate, frameDelayIncrement, buffer_size));
+    frameDelayIncrement = (90000 / frameRateRounded);
+        /* num of "delay" units per frame */
+    bufferFillRate = bit_rate / frameRateRounded; 
+        /* VBV buf fills at constant rate */
+    VBV_buffer = buffer_size;
+    DBG_PRINT(("VBV- delay: %d, fill rate: %d, delay/Frame: %d units, "
+               "buffer size: %d\n",
+               VBV_remainginDelay, bufferFillRate, frameDelayIncrement, 
+               buffer_size));
   
-  result = initGOPRateControl();
+    result = initGOPRateControl();
   
-  return result;
+    return result;
 }
 
 /*===========================================================================*
@@ -317,27 +348,29 @@ int initGOPRateControl _ANSI_ARGS_((void));
  *
  *===========================================================================*/
 int
-  initGOPRateControl()
+initGOPRateControl()
 {
-  DBG_PRINT(("\tInitializing new GOP\n"));
+    DBG_PRINT(("Initializing new GOP\n"));
   
-  Nx = GOP_X;
-  Ni = GOP_I;
-  Np = GOP_P;
-  Nb = GOP_B;
+    Nx = GOP_X;
+    Ni = GOP_I;
+    Np = GOP_P;
+    Nb = GOP_B;
   
-  rc_R += rc_G;
+    rc_R += rc_G;
   
-  DBG_PRINT(("\tbufsize: %d, bitrate: %d, pictrate: %d, GOP bits: %d\n",
-	     buffer_size, bit_rate, frameRateRounded, rc_R));
-  DBG_PRINT(("\tXi: %d, Xp: %d, Xb: %d Nx: %d, Ni: %d, Np: %d, Nb: %d\n",
-	     Xi, Xp, Xb, Nx,Ni,Np,Nb));
-  DBG_PRINT(("\td0_i: %d, d0_p: %d, d0_b: %d, avg_act: %d, rc_Q: %d, mquant: %d\n",
-	     d0_i, d0_p, d0_b, avg_act, rc_Q, mquant));
-  return 1;
+    DBG_PRINT(("bufsize: %d, bitrate: %d, pictrate: %d, GOP bits: %d\n",
+               buffer_size, bit_rate, frameRateRounded, rc_R));
+    DBG_PRINT(("Xi: %d, Xp: %d, Xb: %d Nx: %d, Ni: %d, Np: %d, Nb: %d\n",
+               Xi, Xp, Xb, Nx,Ni,Np,Nb));
+    DBG_PRINT(("d0_i: %d, d0_p: %d, d0_b: %d, avg_act: %d, rc_Q: %d, "
+               "mquant: %d\n",
+               d0_i, d0_p, d0_b, avg_act, rc_Q, mquant));
+    return 1;
 }
 
 
+
 /*===========================================================================*
  *
  * targetRateControl
@@ -351,100 +384,122 @@ int
  *
  *===========================================================================*/
 void
-  targetRateControl(frame)
-MpegFrame   *frame;
-{
-  float temp1, minimumBits;
-  float tempX, tempY, tempZ;
-  int result;
-  int frameType;
-  const char *strPtr;
+targetRateControl(MpegFrame * const frame) {
+
+    float temp1, minimumBits;
+    float tempX, tempY, tempZ;
+    int result;
+    int frameType;
+    const char *strPtr;
   
-  minimumBits = (bit_rate / (8 * frameRateRounded));
+    minimumBits = (bit_rate / (8 * frameRateRounded));
   
-  /*   Check if new GOP */
-  if (Nx == 0) {
-    initGOPRateControl();
-  }
+    /*   Check if new GOP */
+    if (Nx == 0) {
+        initGOPRateControl();
+    }
   
-  if (MB_cnt < 0) {MB_cnt = determineMBCount();}
+    if (MB_cnt < 0) {MB_cnt = determineMBCount();}
   
-  switch (frame->type) {
-  case TYPE_IFRAME:
-    frameType = 'I';
+    switch (frame->type) {
+    case TYPE_IFRAME:
+        frameType = 'I';
     
-    /*		temp1 = ( rc_R / ( 1+ ((Np * Xp) / (Xi * Kp)) + ((Nb*Xb) / (Xi*Kb))))); */
+        tempX = ( (Np * Ki * Xp) / (Xi * Kp) );
+        tempY = ( (Nb * Ki * Xb) / (Xi*Kb) );
+        tempZ = Ni + tempX + tempY;
+        temp1 = (rc_R / tempZ);
+        result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
+        current_Tx = Ti = result;
+        lastFrameVirtBuf = d0_i;
+        break;
     
-    tempX = ( (Np * Ki * Xp) / (Xi * Kp) );
-    tempY = ( (Nb * Ki * Xb) / (Xi*Kb) );
-    tempZ = Ni + tempX + tempY;
-    temp1 = (rc_R / tempZ);
-    result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
-    current_Tx = Ti = result;
-    lastFrameVirtBuf = d0_i;
-    break;
+    case TYPE_PFRAME:
+        frameType = 'P';
+        tempX =  ( (Ni * Kp * Xi) / (Ki * Xp) );
+        tempY =  ( (Nb * Kp * Xb) / (Kb * Xp) );
+        tempZ = Np + tempX + tempY;
+        temp1 = (rc_R/ tempZ);
+        result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
+        current_Tx = Tp = result;
+        lastFrameVirtBuf = d0_p;
+        break;
     
-  case TYPE_PFRAME:
-    frameType = 'P';
-    tempX =  ( (Ni * Kp * Xi) / (Ki * Xp) );
-    tempY =  ( (Nb * Kp * Xb) / (Kb * Xp) );
-    tempZ = Np + tempX + tempY;
-    temp1 = (rc_R/ tempZ);
-    result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
-    current_Tx = Tp = result;
-    lastFrameVirtBuf = d0_p;
-    break;
+    case TYPE_BFRAME:
+        frameType = 'B';
+        tempX =  ( (Ni * Kb * Xi) / (Ki * Xb) );
+        tempY =  ( (Np * Kb * Xp) / (Kp * Xb) );
+        tempZ = Nb + tempX + tempY;
+        temp1 = (rc_R/ tempZ);
+        result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
+        current_Tx = Tb = result;
+        lastFrameVirtBuf = d0_b;
+        break;
     
-  case TYPE_BFRAME:
-    frameType = 'B';
-    tempX =  ( (Ni * Kb * Xi) / (Ki * Xb) );
-    tempY =  ( (Np * Kb * Xp) / (Kp * Xb) );
-    tempZ = Nb + tempX + tempY;
-    temp1 = (rc_R/ tempZ);
-    result = (int) (temp1 > minimumBits ? temp1 :  minimumBits);
-    current_Tx = Tb = result;
-    lastFrameVirtBuf = d0_b;
-    break;
-    
-  default:
-    frameType = 'X';
-  }
+    default:
+        frameType = 'X';
+    }
   
-  N_act = 1;
-  rc_Q = lastFrameVirtBuf  * 31 / reactionParameter;
-  mquant = rc_Q * N_act;
-  Qscale = (mquant > 31 ? 31 : mquant);
-  Qscale = (Qscale < 1 ? 1 : Qscale);
+    N_act = 1;
+    rc_Q = lastFrameVirtBuf  * 31 / reactionParameter;
+    mquant = rc_Q * N_act;
+    Qscale = (mquant > 31 ? 31 : mquant);
+    Qscale = (Qscale < 1 ? 1 : Qscale);
   
-  /*   Print headers for Frame info */
-  strPtr = Frame_header1;
-  DBG_PRINT(("%s\n",strPtr));
-  strPtr = Frame_header2;
-  DBG_PRINT(("%s\n",strPtr));
-  strPtr = Frame_header3;
-  DBG_PRINT(("%s\n",strPtr));
+    /*   Print headers for Frame info */
+    strPtr = Frame_header1;
+    DBG_PRINT(("%s\n",strPtr));
+    strPtr = Frame_header2;
+    DBG_PRINT(("%s\n",strPtr));
+    strPtr = Frame_header3;
+    DBG_PRINT(("%s\n",strPtr));
   
-  /*   Print Frame info */
-  sprintf(rc_buffer, "%4d     %1c  %4d  %6d %7d  %2d %2d %2d   %2.2f  %6d %4d    %3d",
-	  frame->id,frameType,MB_cnt,current_Tx,rc_R,Ni,Np,Nb, N_act, lastFrameVirtBuf, rc_Q, Qscale);
+    /*   Print Frame info */
+    sprintf(rc_buffer, "%4d     %1c  %4d  %6d %7d  "
+            "%2d %2d %2d   %2.2f  %6d %4d    %3d",
+            frame->id,frameType,MB_cnt,current_Tx,rc_R,Ni,Np,Nb, 
+            N_act, lastFrameVirtBuf, rc_Q, Qscale);
   
 #ifdef RC_STATS_FILE
-  fprintf(RC_FILE,"%s\n", rc_buffer);
-  fflush(RC_FILE);
+    fprintf(RC_FILE,"%s\n", rc_buffer);
+    fflush(RC_FILE);
 #endif
-  DBG_PRINT(("%s\n",rc_buffer));
+    DBG_PRINT(("%s\n",rc_buffer));
   
-  /*  Print headers for Macroblock info */
-  if (RC_MB_SAMPLE_RATE) {
-    strPtr = MB_header1;
-    DBG_PRINT(("%s\n",strPtr));
-    strPtr = MB_header2;
-    DBG_PRINT(("%s\n",strPtr));
-  } else {
-    return;
-  }
-  
-  return;
+    /*  Print headers for Macroblock info */
+    if (RC_MB_SAMPLE_RATE) {
+        strPtr = MB_header1;
+        DBG_PRINT(("%s\n",strPtr));
+        strPtr = MB_header2;
+        DBG_PRINT(("%s\n",strPtr));
+    }
+}
+
+
+
+
+static void 
+updateVBVBuffer(int const frameBits) {
+/*----------------------------------------------------------------------------
+  Update the VBV buffer after each frame.  This theoretical buffer is
+  being filled at constant rate 'bufferFillRate' from an mpeg stream.
+  It is emptied as each frame is grabbed by the decoder.  Exception is
+  that the decoder will wait until the "delay" is over.
+
+  We mark the passing of one unit of time, in which 'frameBits' bits of
+  mpeg data were grabbed from the buffer by the decoder.
+-----------------------------------------------------------------------------*/
+    if (VBV_remainingDelay)
+        VBV_remainingDelay -= MIN(frameDelayIncrement, VBV_remainingDelay);
+    else
+        VBV_buffer -= frameBits;
+
+    VBV_buffer += bufferFillRate;
+    if (VBV_buffer < 0 && wantVbvUnderflowWarning)
+        pm_message("WARNING - VBV buffer underflow (%d)", VBV_buffer);
+    if (VBV_buffer > buffer_size && wantVbvOverflowWarning)
+        pm_message("WARNING - VBV buffer overflow (%d > %d)",
+                   VBV_buffer, buffer_size);
 }
 
 
@@ -462,73 +517,76 @@ MpegFrame   *frame;
  *
  *===========================================================================*/
 void
-  updateRateControl(type)
-int type; 
-{
-  int totalBits, frameComplexity, pctAllocUsed, pctGOPUsed;
-  float avgQuant;
-  const char *strPtr;
+updateRateControl(int const type) {
+    int totalBits, frameComplexity, pctAllocUsed, pctGOPUsed;
+    float avgQuant;
+    const char *strPtr;
+
+    totalBits = rc_totalFrameBits;
+    avgQuant = ((float) rc_totalQuant / (float) rc_numBlocks);
+    frameComplexity = totalBits * avgQuant;
+    pctAllocUsed = (totalBits *100 / current_Tx);
+    rc_R -= totalBits;
+    pctGOPUsed = (rc_R *100/ rc_G);
   
-  totalBits = rc_totalFrameBits;
-  avgQuant = ((float) rc_totalQuant / (float) rc_numBlocks);
-  frameComplexity = totalBits * avgQuant;
-  pctAllocUsed = (totalBits *100 / current_Tx);
-  rc_R -= totalBits;
-  pctGOPUsed = (rc_R *100/ rc_G);
+    avg_act = (total_act_j / MB_cnt);
   
-  avg_act = (total_act_j / MB_cnt);
+    updateVBVBuffer(totalBits);
   
-  updateVBVBuffer(totalBits);
-  
-  switch (type) {
-  case TYPE_IFRAME:
-    Ti = current_Tx;
-    d0_i = currentVirtBuf;
-    Ni--;
-    Si = totalBits;
-    Qi = avgQuant;
-    Xi = frameComplexity;
-    break;
-  case TYPE_PFRAME:
-    Tp = current_Tx;
-    d0_p = currentVirtBuf;
-    Np--;
-    Sp = totalBits;
-    Qp = avgQuant;
-    Xp = frameComplexity;
-    break;
-  case TYPE_BFRAME:
-    Tb = current_Tx;
-    d0_b = currentVirtBuf;
-    Nb--;
-    Sb = totalBits;
-    Qb = avgQuant;
-    Xb = frameComplexity;
-    break;
-  }
+    switch (type) {
+    case TYPE_IFRAME:
+        Ti = current_Tx;
+        d0_i = currentVirtBuf;
+        Ni--;
+        Si = totalBits;
+        Qi = avgQuant;
+        Xi = frameComplexity;
+        break;
+    case TYPE_PFRAME:
+        Tp = current_Tx;
+        d0_p = currentVirtBuf;
+        Np--;
+        Sp = totalBits;
+        Qp = avgQuant;
+        Xp = frameComplexity;
+        break;
+    case TYPE_BFRAME:
+        Tb = current_Tx;
+        d0_b = currentVirtBuf;
+        Nb--;
+        Sb = totalBits;
+        Qb = avgQuant;
+        Xb = frameComplexity;
+        break;
+    }
   
   
-  /*  Print Frame info */
-  strPtr = Frame_trailer1;
-  DBG_PRINT(("%s\n",strPtr));
-  strPtr = Frame_trailer2;
-  DBG_PRINT(("%s\n",strPtr));
-  strPtr = Frame_trailer3;
-  DBG_PRINT(("%s\n",strPtr));
+    /*  Print Frame info */
+    strPtr = Frame_trailer1;
+    DBG_PRINT(("%s\n",strPtr));
+    strPtr = Frame_trailer2;
+    DBG_PRINT(("%s\n",strPtr));
+    strPtr = Frame_trailer3;
+    DBG_PRINT(("%s\n",strPtr));
   
-  sprintf(rc_buffer, "%6d  %2.2f  %6d  %3d  %2.2f %7d   %3d %7d   %3d  %6d %6d",
-	  totalBits, avgQuant, frameComplexity, avg_act, N_act, currentVirtBuf, pctAllocUsed, rc_R, pctGOPUsed, VBV_buffer, VBV_delay);
+    sprintf(rc_buffer, "%6d  %2.2f  %6d  %3d  %2.2f %7d   "
+            "%3d %7d   %3d  %6d %6d",
+            totalBits, avgQuant, frameComplexity, avg_act, N_act, 
+            currentVirtBuf, pctAllocUsed, rc_R, pctGOPUsed, 
+            VBV_buffer, VBV_remainingDelay);
 #ifdef RC_STATS_FILE
-  fprintf(RC_FILE,"%s\n", rc_buffer);
-  fflush(RC_FILE);
+    fprintf(RC_FILE,"%s\n", rc_buffer);
+    fflush(RC_FILE);
 #endif
-  DBG_PRINT(("%s\n",rc_buffer));
+    DBG_PRINT(("%s\n",rc_buffer));
   
-  Nx--;
-  rc_totalMBBits= rc_bitsThisMB= rc_totalFrameBits=rc_totalOverheadBits = 0;
-  rc_numBlocks = rc_totalQuant = total_act_j = currentVirtBuf = 0;
+    Nx--;
+    rc_totalMBBits= rc_bitsThisMB= rc_totalFrameBits=rc_totalOverheadBits = 0;
+    rc_numBlocks = rc_totalQuant = total_act_j = currentVirtBuf = 0;
   
-  DBG_PRINT(("GOP now has %d bits remaining (%3d%%) for %d frames .. , Ni= %d, Np= %d, Nb= %d\n", rc_R, (rc_R*100/rc_G), (Ni+Np+Nb), Ni, Np, Nb));
+    DBG_PRINT(("GOP now has %d bits remaining (%3d%%) for %d frames .. , "
+               "Ni= %d, Np= %d, Nb= %d\n", 
+               rc_R, (rc_R*100/rc_G), (Ni+Np+Nb), Ni, Np, Nb));
   
 }
 
@@ -758,7 +816,7 @@ void checkSpatialActivity(blk0, blk1, blk2, blk3)
       P_mean += *(blk_ptr++);
       /*			P_mean += curBlock[j]; 
 				if (curBlock[j] != *(blk_ptr++)) {
-				printf("\n\tARRAY ERROR: block %d\n", j);
+				printf("ARRAY ERROR: block %d\n", j);
 				}
 				*/
     }
@@ -771,7 +829,7 @@ void checkSpatialActivity(blk0, blk1, blk2, blk3)
     for (j=0; j < DCTSIZE_SQ; j++) {
 #ifdef notdef
       if (curBlock[j] != *(blk_ptr++)) {
-	printf("\n\tARRAY ERROR: block %d\n", j);
+	printf("ARRAY ERROR: block %d\n", j);
       }
       temp = curBlock[j] - P_mean;
 #endif      
@@ -925,40 +983,4 @@ int getBufferSize ()
 }
 
 
-/*===========================================================================*
- *
- * updateVBVBuffer ()
- *
- *      Update the VBV buffer after each frame.  This theoretical 
- * buffer is being filled at constant rate, given by the bit rate.
- * It is emptied as each frame is grabbed by the decoder.  Exception 
- * is that the deocder will wait until the "delay" is over.
- *
- * RETURNS:     nothing
- *
- * SIDE EFFECTS:   VBV_buffer
- *
- * NOTES:	
- *
- *===========================================================================*/
-void updateVBVBuffer (frameBits)
-     int frameBits;
-{
-  if (VBV_delay) {
-    VBV_delay -= frameDelayIncrement;
-    if (VBV_delay < 0) {
-      VBV_delay = 0;
-    }
-    
-  } else {
-    VBV_buffer -= frameBits;
-  }
-  VBV_buffer += bufferFillRate;
-  if (VBV_buffer < 0) {
-    fprintf(stderr, "\tWARNING - VBV buffer underflow (%d)\n", VBV_buffer);
-  }
-  if (VBV_buffer > buffer_size) {
-    fprintf(stderr, "WARNING - VBV buffer overflow (%d > %d)\n",
-	    VBV_buffer, buffer_size);
-  }
-}
+
