@@ -11,6 +11,7 @@
 =============================================================================*/
 
 #include <assert.h>
+#include <string.h>
 #include "pam.h"
 #include "dithers.h"
 #include "mallocvar.h"
@@ -143,6 +144,7 @@ parseCommandLine(int argc, char ** argv,
 }
 
 
+
 static struct pam
 makeOutputPam(unsigned int const width,
               unsigned int const height) {
@@ -329,7 +331,7 @@ doHilbert(FILE *       const ifP,
     int *x,*y;
     int sum;
 
-    grays = pnm_readpam(ifP, &graypam, sizeof(graypam));
+    grays = pnm_readpam(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
 
     bitpam = makeOutputPam(graypam.width, graypam.height);
 
@@ -691,7 +693,7 @@ main(int argc, char *argv[]) {
         tuple * bitrow;
         int row;
 
-        pnm_readpaminit(ifP, &graypam, sizeof(graypam));
+        pnm_readpaminit(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
 
         bitpam = makeOutputPam(graypam.width, graypam.height);
         
