@@ -12,6 +12,8 @@
   Contributed to the public domain by its author 2002.05.05.
 -----------------------------------------------------------------------------*/
 
+#include <string.h>
+
 #include "pam.h"
 #include "shhopt.h"
 
@@ -25,7 +27,7 @@ struct cmdlineInfo {
     const char *tupletype;       /* Tuple type for output PAM */
     unsigned int nInput;
         /* The number of input PAMs.  At least 1, at most 16. */
-    char * inputFilespec[MAX_INPUTS];
+    const char * inputFilespec[MAX_INPUTS];
         /* The PAM files to combine, in order. */
 };
 
